"""Command line interface for YesssSMS."""
import argparse
import configparser
import logging
import sys
from datetime import datetime
from functools import wraps
from os import getenv
from os.path import abspath
from os.path import expanduser

from YesssSMS import YesssSMS
from YesssSMS.const import CONFIG_FILE_CONTENT, CONFIG_FILE_PATHS, HELP, VERSION

MAX_MESSAGE_LENGTH_STDIN = 3 * 160


def cli_errors_handled(func):
    """Decorate and handle cli exceptions."""
    # pylint: disable-msg=R0911
    @wraps(func)
    def func_wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except YesssSMS.MissingLoginCredentialsError:
            print("error: no username or password defined (use --help for help)")
            sys.exit(2)
        except YesssSMS.ConnectionError:
            print(
                "error: could not connect to provider. "
                "check your Internet connection."
            )
            sys.exit(3)
        except YesssSMS.AccountSuspendedError:
            print(
                "error: your account was suspended because of 3 failed login attempts. "
                "try again in one hour."
            )
            sys.exit(4)
        except YesssSMS.SMSSendingError:
            print("error: could not send SMS")
            sys.exit(5)
        except YesssSMS.UnsupportedCharsError:
            print("error: message contains unsupported character(s)")
            sys.exit(6)
        except YesssSMS.EmptyMessageError:
            print("error: cannot send empty message.")
            sys.exit(7)
        except CLI.MissingSettingsError:
            print("error: missing settings or invalid settings.")
            sys.exit(8)

    return func_wrapper


class CLI:
    """CLI class for YesssSMS."""

    class MissingSettingsError(ValueError):
        """missing settings."""

    def __init__(self):
        """Init CLI."""
        self.config_files = CONFIG_FILE_PATHS
        self.yessssms = None
        self.message = None
        self.recipient = None
        self.exit_status = self.cli()

    @staticmethod
    def version_info():
        """Display version information."""
        print("yessssms {}".format(YesssSMS("", "").version()))

    @staticmethod
    def print_config_file():
        """Print a sample config file, to pipe into a file."""
        print(CONFIG_FILE_CONTENT, end="")

    @staticmethod
    def parse_args(args):
        """Parse arguments and return namespace."""
        parser = argparse.ArgumentParser(description=HELP["desc"])
        parser.add_argument("-t", "--to", dest="recipient", help=HELP["to_help"])
        parser.add_argument("-m", "--message", help=HELP["message"])
        parser.add_argument("-c", "--configfile", help=HELP["configfile"])
        parser.add_argument("-l", "--login", dest="login", help=HELP["login"])
        parser.add_argument("-p", "--password", dest="password", help=HELP["password"])
        parser.add_argument(
            "-T",
            "--check-login",
            action="store_true",
            default=False,
            help=HELP["check_login"],
        )
        parser.add_argument("--mvno", dest="provider", help=HELP["provider"])
        parser.add_argument(
            "--version", action="store_true", default=False, help=HELP["version"]
        )
        parser.add_argument(
            "--test", action="store_true", default=False, help=HELP["test"]
        )
        parser.add_argument(
            "--print-config-file",
            action="store_true",
            default=False,
            help=HELP["print-config-file"],
        )
        if not args:
            parser.print_help()
            return None

        return parser.parse_args(args)

    @staticmethod
    def read_env_config():
        """Read YESSSSMS environment variables.

        Returns None if password and login are not set.
        """
        # use environment settings if they exist
        env_login = getenv("YESSSSMS_LOGIN", None)
        env_passwd = getenv("YESSSSMS_PASSWD", None)
        conf = None
        if env_login is not None and env_passwd is not None:
            conf = (
                env_login,
                env_passwd,
                getenv("YESSSSMS_RECIPIENT", None),
                getenv("YESSSSMS_PROVIDER", None),
                None,  # custom_provider_urls not supported in env variables yet
            )
        return conf

    def read_config_files(self, config_file):
        """Read config files for settings."""
        if config_file:
            self.config_files.append(config_file)

        parsable_files = []
        for conffile in self.config_files:
            conffile = expanduser(conffile)
            conffile = abspath(conffile)
            parsable_files.append(conffile)

        login = None
        passwd = None
        default_recipient = None
        provider = None
        custom_provider_urls = None

        try:
            config = configparser.ConfigParser()
            config.read(parsable_files)

            login = str(config.get("YESSSSMS", "LOGIN"))
            passwd = str(config.get("YESSSSMS", "PASSWD"))

            if config.has_option("YESSSSMS", "DEFAULT_TO"):
                default_recipient = config.get("YESSSSMS", "DEFAULT_TO")
            if config.has_option("YESSSSMS", "MVNO"):
                provider = config.get("YESSSSMS", "MVNO")
            if config.has_option("YESSSSMS_PROVIDER_URLS", "LOGIN_URL"):
                custom_provider_urls = {
                    "LOGIN_URL": config.get("YESSSSMS_PROVIDER_URLS", "LOGIN_URL"),
                    "LOGOUT_URL": config.get("YESSSSMS_PROVIDER_URLS", "LOGOUT_URL"),
                    "KONTOMANAGER_URL": config.get(
                        "YESSSSMS_PROVIDER_URLS", "KONTOMANAGER_URL"
                    ),
                    "SEND_SMS_URL": config.get(
                        "YESSSSMS_PROVIDER_URLS", "SEND_SMS_URL"
                    ),
                    "WEBSMS_FORM_URL": config.get(
                        "YESSSSMS_PROVIDER_URLS", "WEBSMS_FORM_URL"
                    ),
                }
        except (
            KeyError,
            configparser.NoSectionError,
            configparser.MissingSectionHeaderError,
        ) as ex:
            # only interested in missing settings if custom file is defined
            # else ignore it.
            if config_file:
                print("error: settings not found: {}".format(ex))
                raise self.MissingSettingsError()

        return (login, passwd, default_recipient, provider, custom_provider_urls)

    # inconsistent return (testing), too many branches
    # pylint: disable-msg=R1710,R0912
    @cli_errors_handled
    def cli(self):
        """Handle arguments for command line interface."""
        # config file settings are overridden by env, which are overridden by
        # command line args
        args = self.parse_args(sys.argv[1:])

        if not args:
            return 0

        if args.print_config_file:
            self.print_config_file()
            return 0
        if args.version:
            self.version_info()
            return 0

        (
            login,
            passwd,
            default_recipient,
            provider,
            custom_provider_urls,
        ) = self.read_config_files(args.configfile or None)

        # use environment settings if they exist
        env_settings = self.read_env_config()
        if env_settings:
            logging.debug("using environment variables")
            (
                login,
                passwd,
                default_recipient,
                provider,
                custom_provider_urls,
            ) = env_settings

        if args.provider:
            provider = args.provider

        if args.login and args.password:
            login = args.login
            passwd = args.password

        logging.debug("login: %s", login)
        if provider:
            self.yessssms = YesssSMS(login, passwd, provider=provider)
        else:
            self.yessssms = YesssSMS(
                login, passwd, custom_provider=custom_provider_urls
            )

        if args.check_login:
            valid = self.yessssms.login_data_valid()
            text = ("ok", "") if valid else ("error", "NOT ")
            print("{}: login data is {}valid.".format(text[0], text[1]))
            return 0 if valid else 1

        if args.message == "-":
            message = ""
            for line in sys.stdin:
                message += line
                if len(message) > MAX_MESSAGE_LENGTH_STDIN:
                    break
            # maximum of 3 SMS if pipe is used
            message = message[:MAX_MESSAGE_LENGTH_STDIN]
        else:
            message = args.message

        if args.test:
            message = message or "yessssms (" + VERSION + ") test message at {}".format(
                datetime.now().isoformat()
            )
        recipient = args.recipient or default_recipient or login

        self.recipient = recipient
        self.message = message
        self.yessssms.send(recipient, self.message)
        return 0


def run():
    """Start and run the CLI."""
    CLI()



"""constants for YesssSMS."""
import json
import os

VERSION = json.loads(
    open(os.path.dirname(os.path.realpath(__file__)) + "/version.json").read()
)["version"]
_UNSUPPORTED_CHARS_STRING = "<strong>Achtung:</strong> Ihre SMS konnte nicht \
versendet werden, da sie folgende ungültige Zeichen enthält:"
_LOGIN_ERROR_STRING = "<strong>Login nicht erfolgreich"
_LOGIN_LOCKED_MESS = "Wegen 3 ungültigen Login-Versuchen ist Ihr Account für \
eine Stunde gesperrt."
_LOGIN_LOCKED_MESS_ENG = "because of 3 failed login-attempts, your account \
has been suspended for one hour"
_UNSUPPORTED_CHARS_STRING = "<strong>Achtung:</strong> Ihre SMS konnte nicht \
versendet werden, da sie folgende ungültige Zeichen enthält:"
_SMS_SENDING_SUCCESSFUL_STRING = ">Ihre SMS wurde erfolgreich verschickt!<"

_SMS_FORM_ID = "smsform"
_SMS_FORM_ID_VALUE = "value"

PROVIDER_TLDS = {
    "billitel": "billitel.kontomanager.at",
    "fenercell": "fenercell.kontomanager.at",
    "educom": "educom.kontomanager.at",
    "goood": "goood.kontomanager.at",
    "kronemobile": "kronemobile.kontomanager.at",
    "georg": "kundencenter.georg.at",
    "kuriermobil": "kuriermobil.kontomanager.at",
    "simfonie": "simfonie.kontomanager.at",
    "teleplanet": "teleplanet.kontomanager.at",
    "wowww": "wowww.kontomanager.at",
    "yooopi": "yooopi.kontomanager.at",
}

PROVIDER_URLS = {
    "yesss": {
        "LOGIN_URL": "https://www.yesss.at/kontomanager.at/index.php",
        "LOGOUT_URL": "https://www.yesss.at/kontomanager.at/index.php?dologout=2",
        "KONTOMANAGER_URL": "https://www.yesss.at/kontomanager.at/kundendaten.php",
        "WEBSMS_FORM_URL": "https://www.yesss.at/kontomanager.at/websms.php",
        "SEND_SMS_URL": "https://www.yesss.at/kontomanager.at/websms_send.php",
    },
    "billitel": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["billitel"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(
            PROVIDER_TLDS["billitel"]
        ),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["billitel"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["billitel"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["billitel"]),
    },
    "educom": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["educom"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(PROVIDER_TLDS["educom"]),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["educom"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["educom"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["educom"]),
    },
    "fenercell": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["fenercell"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(
            PROVIDER_TLDS["fenercell"]
        ),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["fenercell"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["fenercell"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["fenercell"]),
    },
    "georg": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["georg"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(PROVIDER_TLDS["georg"]),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(PROVIDER_TLDS["georg"]),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["georg"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["georg"]),
    },
    "goood": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["goood"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(PROVIDER_TLDS["goood"]),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(PROVIDER_TLDS["goood"]),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["goood"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["goood"]),
    },
    "kronemobile": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["kronemobile"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(
            PROVIDER_TLDS["kronemobile"]
        ),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["kronemobile"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["kronemobile"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(
            PROVIDER_TLDS["kronemobile"]
        ),
    },
    "kuriermobil": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["kuriermobil"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(
            PROVIDER_TLDS["kuriermobil"]
        ),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["kuriermobil"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["kuriermobil"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(
            PROVIDER_TLDS["kuriermobil"]
        ),
    },
    "simfonie": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["simfonie"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(
            PROVIDER_TLDS["simfonie"]
        ),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["simfonie"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["simfonie"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["simfonie"]),
    },
    "teleplanet": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["teleplanet"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(
            PROVIDER_TLDS["teleplanet"]
        ),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["teleplanet"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["teleplanet"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(
            PROVIDER_TLDS["teleplanet"]
        ),
    },
    "wowww": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["wowww"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(PROVIDER_TLDS["wowww"]),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(PROVIDER_TLDS["wowww"]),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["wowww"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["wowww"]),
    },
    "yooopi": {
        "LOGIN_URL": "https://{}/index.php".format(PROVIDER_TLDS["yooopi"]),
        "LOGOUT_URL": "https://{}/index.php?dologout=2".format(PROVIDER_TLDS["yooopi"]),
        "KONTOMANAGER_URL": "https://{}/kundendaten.php".format(
            PROVIDER_TLDS["yooopi"]
        ),
        "WEBSMS_FORM_URL": "https://{}/websms.php".format(PROVIDER_TLDS["yooopi"]),
        "SEND_SMS_URL": "https://{}/websms_send.php".format(PROVIDER_TLDS["yooopi"]),
    },
}

HELP = {
    "to_help": "Recipient phone number in the format: +436601234567",
    "desc": """Send an SMS via the yesss.at website. Some MVNOs that use
          the kontomanager.at interface are also supported.""",
    "configfile": "Path of a config-file. Default paths are: \
          '/etc/yessssms.conf' and '~/.config/yessssms.conf'. \
          An example file is yessssms_sample.conf.",
    "login": "Your phone number (eg. 06501234567), used to login at \
          yesss.at.\
          Or use the environment variable 'YESSSSMS_LOGIN'.",
    "password": """Your password, it\'s not recommended to use this. \
          Use a config-file instead (see: -c, --configfile).\
          Or use the environment variable 'YESSSSMS_PASSWD'.""",
    "check_login": """validates your login data""",
    "provider": """set a MVNO (mobile virtual network operator) other \
          than Yesss.at (default)\
          available are: {}""".format(
        ", ".join(list(PROVIDER_URLS.keys()))
    ),
    "message": "Message to be sent by SMS",
    "version": "print version information.",
    "test": "send a test message to yourself",
    "print-config-file": "prints a sample config file, that can be piped \
          into eg. ~/.config/yessssms.conf.",
}
CONFIG_FILE_CONTENT = """# place this file, with correct credentials, at /etc/yessssms.conf
# or ~/.config/yessssms.conf
[YESSSSMS]
LOGIN =  06501234567
PASSWD = MySecre3tPassw0rd
# you can define a default recipient (will be overridden by -t option)
# DEFAULT_TO = +43664123123123

# define alternative MVNO (YESSS, EDUCOM, etc. see --help (--mvno))
# MVNO = YESSS

# # very advanced settings (be warned!!):
# [YESSSSMS_PROVIDER_URLS]

# # eg: EDUCOM:
# LOGIN_URL = https://educom.kontomanager.at/index.php
# LOGOUT_URL = https://educom.kontomanager.at/index.php?dologout=2
# KONTOMANAGER_URL = https://educom.kontomanager.at/kundendaten.php
# WEBSMS_FORM_URL = https://educom.kontomanager.at/websms.php
# SEND_SMS_URL = https://educom.kontomanager.at/websms_send.php
"""
# CONFIG_FILE_PATHS = []
CONFIG_FILE_PATHS = ["/etc/yessssms.conf", "~/.config/yessssms.conf"]

TEST_FORM_TOKEN_SAMPLE = (
    "<form action='websms_send.php' name='sms' id='smsform'"
    " method='post' onSubmit=\"return validate()\">"
    '<input type="hidden" name="token" value="f2ca1bb6c7e907d06dafe4687e579fc'
    "e76b37e4e93b7605022da52e6ccc26fd2\"><div class='form-group'>"
    "    <div class='input-row'>"
)
